/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2000 - 2010, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.authordoclet.reader;

import org.jboss.authordoclet.Constants;
import org.jboss.authordoclet.Context;
import org.jboss.authordoclet.anchor.Anchor;
import org.jboss.authordoclet.util.XHTMLParser;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 *
 */
public abstract class AbstractReader implements Reader {

    final private XHTMLParser xhtmlParser = XHTMLParser.newInstance();

    public XHTMLParser getXhtmlParser() {
        return xhtmlParser;
    }

    protected Element readTitle(Anchor citation) {
        return citation.getTitle() != null ?
                getXhtmlParser().wrap(
                        Constants.WRAPPER_ELEMENT_NAME,
                        new String[][]{new String[]{Constants.ATTR_TYPE, Constants.TYPE_TITLE}},
                        citation.getTitle()
                ).getDocumentElement()
                : null;
    }    

    protected Document wrap(Anchor citation, Element... elements) {
        return getXhtmlParser().wrap(
                Constants.WRAPPER_ELEMENT_NAME,
                new String[][]{
                        new String[]{Constants.ATTR_TYPE, citation.getOutputClasses()},
                        new String[]{Constants.ATTR_ID, citation.getOutputIdentifier()}
                },
                elements
        );
    }

    protected String getAbsolutePath(Anchor citation, Context context, String contextVarName) {
        StringBuilder absolutePath = new StringBuilder();
        absolutePath.append(context.get(contextVarName));
        if (absolutePath.length() > 0 && !absolutePath.toString().endsWith("/"))
            absolutePath.append("/");
        absolutePath.append(citation.getAddress().getPath());
        return absolutePath.toString();
    }

}
