/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2000 - 2010, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.authordoclet.util;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.Source;
import java.io.InputStream;

/**
 *
 */
public abstract class XHTMLParser extends DOMParser {

    public static final String XHTML_NAMESPACE_URI = "http://www.w3.org/1999/xhtml";
    public static final String XHTML_SCHEMA_RESOURCE = "org/jboss/authordoclet/schemas/xhtml1-strict.xsd";

    public static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";

    protected Source[] getSchemaSources() {
        InputStream xhtmlSchema = Thread.currentThread().getContextClassLoader().getResourceAsStream(XHTML_SCHEMA_RESOURCE);
        return new Source[]{new StreamSource(xhtmlSchema)};
    }

    public static XHTMLParser newInstance() {
        return new XHTMLParser() {
            public void error(SAXParseException e) throws SAXException {
                throw new RuntimeException("Error parsing XHTML: " + e, e);
            }

            public void fatalError(SAXParseException e) throws SAXException {
                throw new RuntimeException("Error parsing XHTML: " + e, e);
            }
        };
    }
}
